@@ -133,6 +133,14 @@ class cmd_explorer(Command):
 
     def run(self, location=None, hat=None, dry_run=False,
             desktop=None, zzz=False, experimental=False, screenshots=False):
+        
+        required_qbzr_version = (0, 19)
+        from bzrlib.plugins import qbzr
+        if qbzr.version_info < required_qbzr_version:
+            from bzrlib.errors import IncompatibleAPI
+            raise IncompatibleAPI(qbzr, required_qbzr_version,
+                                  qbzr.version_info, qbzr.version_info)
+        
         from bzrlib.plugins.qbzr.lib import uifactory
         _init_explorer_command(zzz, experimental)
 
