@@ -81,8 +81,8 @@ from bzrlib.plugins.qbzr.lib.util import (
 
 _LP_PROJECT = "bzr-explorer"
 _HELP_URL = "http://doc.bazaar.canonical.com/explorer/en/documentation.html"
-_CODE_NAME = "Marco Polo"
-_CODE_NAME_URL = "http://en.wikipedia.org/wiki/Marco_Polo"
+_CODE_NAME = "Bartolomeu Dias"
+_CODE_NAME_URL = "http://en.wikipedia.org/wiki/Bartolomeu_Dias"
 
 
 _MSG_TYPE_TO_ICON = {
@@ -196,6 +196,9 @@ class QExplorerMainWindow(QBzrWindow):
             self._do_view_action, self.ui.action_Browse,
             gettext("Working Tree"))
         self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self._wt_browser)
+        # Set up windows array used by QBzr TreeWidget when working tree
+        # browser is undocked
+        self._wt_browser.windows = self.windows
 
         # Load the bookmarks and tools menus
         self.init_hats()
@@ -599,9 +602,11 @@ class QExplorerMainWindow(QBzrWindow):
         available = self._supported_commands
         kind = None
         working_tree = None
+        wt_applicable = False
         if view:
             kind = view.kind()
-            if kind != kinds.WELCOME_PAGE:
+            wt_applicable = view.has_wt()
+            if view.supports_wt():
                 working_tree = view.working_tree()
         for action, conditions in self._conditions_by_action.items():
             try:
@@ -625,11 +630,6 @@ class QExplorerMainWindow(QBzrWindow):
         # Update the working tree browser, hiding it for pages where
         # it's never applicable
         action = self.ui.action_WorkingTree_Browser
-        wt_applicable = working_tree is not None and kind in [
-            kinds.BRANCH_KIND,
-            kinds.BOUND_BRANCH_KIND,
-            kinds.CHECKOUT_KIND,
-            ]
         action.setEnabled(wt_applicable)
         wt_visible = wt_applicable and action.isChecked()
         self._wt_browser.setVisible(wt_visible)
@@ -748,7 +748,7 @@ class QExplorerMainWindow(QBzrWindow):
             if value != default_value:
                 self._preference_changed(name, value, old_value=value)
 
-    def _preference_changed(self, name, new_value, old_value=None):
+    def _preference_changed(self, name, new_value, old_value=None, parent=None):
         """Handle the change of a preference."""
         trace.mutter("setting preference %s to %s (was %s)" %
             (name, new_value, old_value))
@@ -762,7 +762,7 @@ class QExplorerMainWindow(QBzrWindow):
             self._apply_workingtree_style(new_value)
         elif name == 'location-selector-style':
             if new_value != old_value:
-                self.show_restart_required()
+                self.show_restart_required(parent)
         elif name == 'toolbox-style':
             if new_value != old_value:
                 self.show_restart_required()
@@ -770,6 +770,9 @@ class QExplorerMainWindow(QBzrWindow):
             self._update_toolbox_visibility()
         elif name == 'workingtree-default-to-edit':
             self._apply_workingtree_default_to_edit(new_value)
+        elif name == 'language':
+            if new_value != old_value:
+                self.show_restart_required(parent)
 
     def _apply_app_suite(self, app_suite):
         try:
@@ -946,20 +949,26 @@ class QExplorerMainWindow(QBzrWindow):
         else:
             args = mod_app_suite.command_to_args(tool.action,
                 self._escape_context(self.location_context))
-            if tool.type == 'bzr':
+            if (tool.type == 'bzr') or (tool.type == 'bzr-exec'):
                 # If the command looks like it's a GUI one,
                 # run it directly. Otherwise, run it via qrun
                 # making sure that we're passing --ui-mode.
                 if args[0][0] in ['q', 'g']:
                     args = self._get_bzr_executable() + args
+                elif tool.type == 'bzr-exec':
+                    args = self._get_bzr_executable () + \
+                           ['qrun', '--ui-mode', '--execute', '--'] + args
                 else:
-                    args = self._get_bzr_executable() + ['qrun', '--ui-mode', '--'] + args
-            self.run_app(args)
+                    args = self._get_bzr_executable() + \
+                           ['qrun', '--ui-mode', '--'] + args
+            self.run_app(args, use_shell=(tool.type=='shell'))
 
     ## Show message helpers ##
 
-    def _message(self, type, title, message, extra_info=None, details=None):
-        mbox = QtGui.QMessageBox(self)
+    def _message(self, type, title, message, extra_info=None, details=None, parent=None):
+        if parent is None:
+            parent = self
+        mbox = QtGui.QMessageBox(parent)
         mbox.setIcon(_MSG_TYPE_TO_ICON[type])
         mbox.setWindowTitle(gettext(title))
         mbox.setText(gettext(message))
@@ -969,21 +978,22 @@ class QExplorerMainWindow(QBzrWindow):
             mbox.setDetailedText(details)
         mbox.exec_()
 
-    def show_note(self, title, message, extra_info=None, details=None):
-        self._message("note", title, message, extra_info, details)
+    def show_note(self, title, message, extra_info=None, details=None, parent=None):
+        self._message("note", title, message, extra_info, details, parent)
 
-    def show_warning(self, title, message, extra_info=None, details=None):
-        self._message("warning", title, message, extra_info, details)
+    def show_warning(self, title, message, extra_info=None, details=None, parent=None):
+        self._message("warning", title, message, extra_info, details, parent)
 
-    def show_error(self, title, message, extra_info=None, details=None):
-        self._message("error", title, message, extra_info, details)
+    def show_error(self, title, message, extra_info=None, details=None, parent=None):
+        self._message("error", title, message, extra_info, details, parent)
 
     def show_not_available_yet(self):
         self.show_note("Sorry", u"Action is not available yet.")
 
-    def show_restart_required(self):
-        self.show_note(gettext("Restart Required"), gettext(
-            "Please restart Bazaar Explorer for this preference to take effect."))
+    def show_restart_required(self, parent=None):
+        self.show_warning(gettext("Restart Required"), gettext(
+            "Please restart Bazaar Explorer for this preference to take effect."),
+            parent=parent)
 
     ## Request input helpers ##
 
@@ -1060,7 +1070,8 @@ class QExplorerMainWindow(QBzrWindow):
                 return None
         return self._hosted_url_resolver.get_branch_url('.')
 
-    def run_app(self, args, refresh_on_success=False, success_callback=None):
+    def run_app(self, args, refresh_on_success=False, success_callback=None,
+                use_shell=False):
         """Run an external application.
 
         :param args: list of arguments. The first one is the command name
@@ -1069,6 +1080,8 @@ class QExplorerMainWindow(QBzrWindow):
           and the current page on process start is still current, refresh it
         :param success_callback: callable to invoke if the command completes
           successfully; if not None, overrides refresh_on_success
+        :param use_shell: if True, cause the subprocess to use a shell
+          to invoke args.
         """
         # In diagnostic mode, just display what we would have done
         if self.dry_run:
@@ -1088,14 +1101,46 @@ class QExplorerMainWindow(QBzrWindow):
                 if self.current_view() == view_on_launch:
                     self.do_refresh()
             success_callback = conditional_refresh
-        self._app_runner.run_app(args, success_callback)
+        self._app_runner.run_app(args, success_callback, use_shell=use_shell)
 
     def _get_bzr_executable(self):
-        if getattr(sys, 'frozen', None) is not None:
-            # Being run from a compiled executable
-            return [sys.argv[0]]
-        # This should be 'python path/to/bzr'
-        return [sys.executable, sys.argv[0]]
+        # XXX this is copied form qbzr/lib/subprocess. Try refactor so that
+        # we have code reuse.
+        if getattr(sys, "frozen", None) is not None:
+            bzr_exe = sys.executable
+            possible_exes = ("bzrw.exe", "bzr.exe")
+            if os.path.basename(bzr_exe) in possible_exes:
+                return [bzr_exe]
+            else:
+                # Was run from something like tbzrcommand.
+                for possible_exe in possible_exes:
+                    bzr_exe = os.path.join(os.path.dirname(sys.executable),
+                                           possible_exe)
+                    if os.path.isfile(bzr_exe):
+                        return [bzr_exe]
+                raise errors.BzrError(gettext('Could not locate "bzr.exe".'))
+        else:
+            # otherwise running as python script.
+            # ensure run from bzr, and not others, e.g. tbzrcommand.py
+            script = sys.argv[0]
+            # make absolute, because we may be running in a different
+            # dir.
+            script = os.path.abspath(script)
+            if os.path.basename(script) == "bzr":
+                return [sys.executable, script]
+            else:
+                import bzrlib
+                # are we running directly from a bzr directory?
+                script = os.path.join(bzrlib.__path__[0], "..", "bzr")
+                if os.path.isfile(script):
+                    return [sys.executable, script]
+                # maybe from an installed bzr?
+                script = os.path.join(sys.prefix, "scripts", "bzr")
+                if os.path.isfile(script):
+                    return [sys.executable, script]
+                
+                raise errors.BzrError(
+                    gettext('Could not locate "bzr" script.'))
 
     def run_bzr_gui(self, cmd_id, context=None):
         """Run a bzr GUI application.
@@ -1681,7 +1726,10 @@ class QExplorerMainWindow(QBzrWindow):
 
     def do_refresh(self, reopen=False):
         self.location_viewer.refresh_current(reopen=reopen)
-        self._wt_browser.refresh()
+        # only refresh working tree if current view has one
+        view = self.current_view()
+        if view and view.has_wt():
+            self._wt_browser.refresh()
         self.changed_view()
 
     def refresh_diagnostic_mode(self):
